<?php
$test_it_path = realpath(dirname(__FILE__));
include_once($test_it_path . '/simpletestauto_settings.php');

chdir(realpath(dirname(__FILE__) . '/../../'));
if (file_exists('./includes/bootstrap.inc')) {
  include_once './includes/bootstrap.inc';
}
else {
  chdir(getcwd() . '/../../');
  if (file_exists('./includes/bootstrap.inc')) {
    include_once './includes/bootstrap.inc';
  }
  else {
  	exit("bootstrap.inc could not be loaded\n");
  }
}

/**
 * Main test script. Call this with node id of the test_instance
 * @return TRUE if test passes FALSE if fails
 */
function test($node_id) {
  global $simpletest_install_dir, $simpletest_modules, $bins, $msg, $executed_commands, $install_url;
  
  // we get all the data we need through a xmlrpc call to main server where data is held
  $node = xmlrpc(variable_get('simpletestauto_return_address', ''), 'simpletestauto_getTestData', sha1(variable_get('simpletestauto_server_pwd', '')), $node_id);

  $patch_url = $node['patch_url'];
  $version = getVersion($node['version']);
  if (!$version) return terminateTesting(1, "$version is not yet supported");

  $msg = "";
  $executed_commands = array();
  $module_dir = drupal_get_path('module', 'simpletestauto');
  $install_dir = $simpletest_install_dir;

  // Generate a unique instance name.
  $instance = "test_" . time() . rand(0, 32767);
  $instance_dir = $install_dir."/".$instance;
  $install_url = $simpletest_install_dir_url . $instance;

  if (!chdir($install_dir)) return terminateTesting(1, "Unable to chdir to $install_dir");
  if (!mkdir($instance_dir)) return terminateTesting(1, "Unable to mkdir $instance_dir\n");
  if (!mkdir($instance_dir.'/files', 0777)) return terminateTesting(1, "Unable to crate files dir\n");
  if (!mkdir($instance_dir.'/sites/all/modules', 0777)) return terminateTesting(1, "Unable to crate modules dir\n");

  //Check out drupal core from cvs
  $exec_cmd = $bins['cvs'] . ' -z6 -Q -d:pserver:anonymous:anonymous@cvs.drupal.org:/cvs/drupal co -d' . $instance .' -r ' . $version['min'] . ' drupal';
  if (!execute($exec_cmd, "Unable to check Drupal branch " . $version['min'] . " out of CVS.")) return FALSE;

  if (!chdir($instance_dir)) return terminateTesting(1, "Unable to chdir to $instance_dir");

  //Check out needed modules
  foreach($simpletest_modules as $module) {
  	$module_tag = get_latest_tag($module, $version);
  	if (!$module_tag) return terminateTesting(1, "Could not get tag for module $module core version " . $version['core']);
    $exec_cmd = $bins['cvs'] . ' -z6 -Q -d:pserver:anonymous:anonymous@cvs.drupal.org:/cvs/drupal-contrib co -d sites/all/modules/'.$module.' -r ' . $module_tag . ' contributions/modules/' . $module;
    if (!execute($exec_cmd, "Unable to check $module module branch " . $version['maj'] . " out of CVS.")) return FALSE;
  }

  //applying patch
  $exec_cmd = $bins['wget'] . ' -q -O - '.$patch_url.' | ' . $bins['patch'] . ' -p0 -d ' . $instance_dir;
  if (!execute($exec_cmd, "Unable to patch file.")) return FALSE;
  
  if (!checkSyntax($patch_url)) return FALSE; 

  return terminateTesting(0, $msg);
}

/**
 * Function that sends response to test server and calls exit
 * @param exit_status arguemnt for exit call
 * @param msg message string for sending to test server 
 */
function terminateTesting($exit_status, $terminate_msg) {
  global $drupal_dir, $node_id, $executed_commands, $debug, $test_it_path, $msg, $install_url;
  fopen('clean','w');
  chdir($drupal_dir);
  $msg = $msg . "<br>" . $terminate_msg;
  if ($debug) {
    $commands = implode("<br>", $executed_commands);
    $msg = "<br>Executed commands:<br>" . $commands . "<br>" . $msg ;
  } 
  $msg = xmlrpc(variable_get('simpletestauto_return_address', ''), 'simpletestauto.writeResult', sha1(variable_get('simpletestauto_server_pwd', '')), $node_id, $msg, $exit_status, $install_url);
  if ($exit_status == 1) {
    return FALSE;
  }
  else {
    return TRUE;
  }
}


/**
 * Checks if all files in array are where they are supposed to be. If not terminates
 * testing with a proper msg
 * @param $bins array where value is path to the file
 */
function check_bins($bins) {
  $not_found = array();
  foreach ($bins as $key => $value) {
  	if (!file_exists($value)) {
  	  $not_found[] = $value;
  	}
  }
  if (count($not_found) != 0) {
    exit("Following binaries were not found in their paths: " . implode(",",$not_found) . "\n");
  }
}


/**
 * Executes command
 * @param $cmd command to execute
 * @param $fail_msg msg to be displayed on fail
 */
function execute($cmd, $fail_msg) {
  global $executed_commands, $msg, $debug;
  $executed_commands[] = $cmd;
  exec($cmd, $output, $status);
  if ($status) {
    return terminateTesting(1, "<strong>" . $fail_msg . "<br>" . implode($output, "<br>")  . "</strong>");
  }
  if (!empty($output)) {
    $msg .= "<br>Command: " . $cmd ."<br> Result:<br>";
    $msg .= "<strong>" . implode($output, "<br>") . "</strong>";
  }
  return TRUE;
}

/**
 * Returns Drupal major version. Basically this function strips down minor Drupal versions.
 * @return major drupal version
 */
function getVersion($v) {
  if ($v == '6.x-dev') {
    $version['min'] = 'HEAD';
    $version['maj'] = 'HEAD';
    $version['core'] = 'HEAD';
  }
  else if ($v = '6.0-beta1') {
    $version['maj'] = 'DRUPAL-6-0-BETA-1';
    $version['min'] = 'DRUPAL-6-0-BETA-1';
    $version['core'] = 'HEAD';
  }
  else {
    $version = FALSE;
  }
  return $version;
}

/**
 * Checks http://updates.drupal.org/release-history/ of specific module and gets latest
 * stable or unstable if stable does not exists for specific core version
 * 
 * @param $module module name which we want to checkout
 * @version $version version of core we are after
 * @return cvs tag ready for checkout
 */
function get_latest_tag($module, $version) {
  global $release_url, $msg;
  if ($version['core'] == 'HEAD') {
    return 'HEAD';
  }
  $module_url = $release_url . $module . "/" . $version['core'];
  $data = array();
  $latest_release = null;
  $xml = drupal_http_request($module_url);
  if (isset($xml->data)) {
    $data = new SimpleXMLElement($xml->data);
  }
  if (!isset($data->releases)) {
    $msg .= (string) $data;
    return FALSE;
  }
  foreach ($data->releases->release as $release) {
    if (isset($latest_release) && ((int) $latest_release->date < (int) $release->date) && !isset($release->version_extra)) {
      $latest_release = $release;
    }
    else if (!isset($latest_release)) {
      $latest_release = $release;
    }
  }
  $tag = (string) $latest_release->tag;
  return $tag;
}

/**
 * Checky syntax of files that are patched with patch at patch_url.
 * Syntax is checked with php -l 
 *
 * @param $patch_url patch url from which files that need checking are read
 */
function checkSyntax($patch_url) {
  global $bins;
  $patch = file_get_contents($patch_url, FALSE);
  $line = strtok($patch, "\n");
  $test_status = TRUE;
  $msg = '';
  while ($line !== false) {
    if(preg_match('/^(Index: )(.*)/',$line, $matches)) {
      $files_to_check[] = $matches[2];
    }
    $line = strtok("\n");
  }
  foreach($files_to_check as $i => $file) {
    $exec_cmd = $bins['phpExec'] . " -l -f $file";
    $output = null;
    exec($exec_cmd, $output, $status);
    if ($status != 0) {
      $test_status = FALSE;
      $msg .= implode($output, "<br>");
    }
  }
  if (!$test_status) {
    return terminateTesting(1, '<strong><br>Syntax error: <br>' . $msg . '</strong>'); 
  }
  return $test_status;
}

$drupal_dir = getcwd();
drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);

if (!variable_get('simpletest_in_progress', 0)) {
  variable_set('simpletest_in_progress', 1);
}
else {
  exit("Test already in progress\n");
}

check_bins($bins);

$current_tests = variable_get('simpletestauto_tests', array());
variable_set('simpletestauto_tests', array());
if (empty($current_tests)) {
  variable_set('simpletest_in_progress', 0);
  exit("No tests to run\n");
}

while(!empty($current_tests)) {
  foreach($current_tests as $test => $node_id) {
    if (is_numeric($node_id)) {
      test($node_id);
    }
  }
  // ugly hack to get new variables from db
  $conf = variable_init();
  $current_tests = $conf['simpletestauto_tests'];
  variable_set('simpletestauto_tests', array());
}
variable_set('simpletest_in_progress', 0);
exit("Testing finished\n");



?>
